import ReactNativeZoomableView from "@dudigital/react-native-zoomable-view/src/ReactNativeZoomableView";
import JitsiMeetJS from "lib-jitsi-meet/lib-jitsi-meet.min";
import React, { Component } from "react";
import {
  AppState,
  Dimensions,
  FlatList,
  Image,
  NativeEventEmitter,
  NativeModules,
  Platform,
  Text,
  TouchableOpacity,
  View,
} from "react-native";
import { RTCView } from "react-native-webrtc";
import AudioMode from "./AudioMode";
import AudioButton from "./components/AudioButton";
import AudioLevelIndicator from "./components/AudioLevelIndicator.native";
import AudioModeButton from "./components/AudioModeButton";
import HangUpButton from "./components/HangUpButton";
import SwitchCameraButton from "./components/SwitchCameraButton";
import TextAvatar from "./components/TextAvatar";
import VideoButton from "./components/VideoButton";
import Logger from "./lib/Logger";
import RTCManager from "./lib/RTCManager";
import localStyles from "./CallingComponentStyles";
import { isScreenSharingTrack } from "./lib/utils";
import {
  SCREEN_SHARING_STARTED,
  SCREEN_SHARING_STOPPED,
  initOptions,
  confOptions,
  singleMode,
  spotlightMode,
  options,
  constChannelLastN,
} from "./util/Constants";
const { CometChatRTC } = NativeModules;

let roomName = "Conference";
let connection = null;
let room = null;
let isJoined = false;

let channelLastN = constChannelLastN;

const USER_AVATAR_URL = "avatar-url";
const USER_UID = "uid";
const USER_JOINED_AT = "joined_at";

const getTotalVideoStreams = (users) => {
  let count = 0;
  users.forEach((user) => {
    if (!user.isVideoMuted) {
      count++;
    }
  });
  return count;
};

const { height, width } = Dimensions.get("window");

var eventEmitter = new NativeEventEmitter(AudioMode);

let TAG = "CallingComponent";
export default class CallingComponent extends Component {
  constructor(props) {
    super(props);
    this.participantsWhoPinnedMe = new Set();

    const localUser = {
      name: this.props.username || "Unnamed",
      avatar: this.props.RTCUser.avatar || "No avatar",
      status: "active",
      isVideoMuted: false,
      isAudioMuted: false,
      isLocalUser: true,
    };

    this.state = {
      mySessionId: this.props.sessionID,
      myUserName: this.props.username,
      status: "active",
      myAvatar: this.props.avatar,
      remoteTracks: [],
      users: [localUser],
      dominantSpeakerId: null,
      isScreenShareOn: false,
      pinnedUserId: null,
      hideAudioMuteButton: this.props.hideAudioMuteButton,
      hidePauseVideoButton: this.props.hidePauseVideoButton,
      hideEndCallButton: this.props.hideEndCallButton,
      hideVideoSwitchButton: this.props.hideVideoSwitchButton,
      hideAudioModeButton: this.props.hideAudioModeButton,
      showButtonLayout: this.props.enableDefaultLayout,
      showUserPopUP: false,
      hidePeopleLable: false,
      hideUserThumbList: true,
      isVideoMirrored: true,
      isSingleMode: false,
      activeUserIds: [],
      mode: this.props.mode,
      region: this.props.REGION,
      isAudioOnly: this.props.isAudioOnly,
      showLocalStream: false,
      appState: AppState.currentState,
      audioMode: "speaker",
      isVideoMuted: false,
      isAudioMuted: false,
      isPortrait: true,
      showUserList: false,
      resolutions: {},
    };
  }

  componentDidMount() {
    CometChatRTC.log("componentDidMount" + this._handleAppStateChange);
    AppState.addEventListener("change", this._handleAppStateChange);
    this.setModeProps(this.props.mode);
    this.createLocalTracks();
    this.init(this.state.mySessionId);
    JitsiMeetJS.setLogLevel(JitsiMeetJS.logLevels.ERROR);
    Logger.log(TAG, "Component did mount ", "");
    this.onAudioModeChanged = this.onAudioModeChanged.bind(this);
    this.onDoubleTap = this.onDoubleTap.bind(this);
    this.renderUserItem = this.renderUserItem.bind(this);
    this.onAudioMute = this.onAudioMute.bind(this);
    this.onVideoMute = this.onVideoMute.bind(this);
    this.onHangUp = this.onHangUp.bind(this);
    this.addNativeEventListner();
    this.initExposedfunctions();

    Dimensions.addEventListener("change", () => {
      this.isPortrait();
    });
  }

  isPortrait = () => {
    const dim = Dimensions.get("screen");
    this.setState({ isPortrait: dim.height >= dim.width });
  };

  componentWillUnmount() {
    CometChatRTC.log("componentWillUnmount" + this._handleAppStateChange);
    //if(this._handleAppStateChange){
    AppState.removeEventListener("change", this._handleAppStateChange);
    //}
  }

  get localUser() {
    return this.state.users[0];
  }
  dominantSpeakers = [];

  _handleAppStateChange = (nextAppState) => {
    Logger.log(TAG, "_handleAppStateChange called....", nextAppState);
    if (
      this.state.appState.match(/inactive|background/) &&
      nextAppState === "active"
    ) {
      this.stopScreenShare();
    }
    this.setState({ appState: nextAppState });
  };

  initExposedfunctions() {
    RTCManager.initOnEndCall(this.unload);
    RTCManager.initSwitchCamera(this.onDoubleTap);
    RTCManager.initOnMuteAudio(this.muteAudio);
    RTCManager.initOnUnMuteAudio(this.unmuteAudio);
    RTCManager.initOnPauseVideo(this.pauseVideo);
    RTCManager.initOnUnPauseVideo(this.unPauseVideo);
  }

  unload = (e) => {
    const { localUser } = this;
    CometChatRTC.log("Local User = " + localUser);
    if (localUser && localUser.audioTrack) {
      try {
        localUser.audioTrack.dispose();
      } catch (error) {
        CometChatRTC.log("Audio dispoce error = " + error);
      } finally {
        CometChatRTC.log("Audio dispoce error finally");
      }
    }
    if (localUser && localUser.videoTrack) {
      try {
        localUser.videoTrack.dispose();
      } catch (error) {
        CometChatRTC.log("Video dispoce error = " + error);
      } finally {
        CometChatRTC.log("Video dispoce error finally");
      }
    }

    try {
      if (room) room.leave();
    } catch (error) {
      CometChatRTC.log("Leave room error = " + error);
    }

    if (connection) connection.disconnect();

    AppState.removeEventListener("change", this._handleAppStateChange);
  };

  setModeProps = (mode) => {
    Logger.log(TAG, "Mode = ", mode);

    if (mode == singleMode) {
      channelLastN = 1;
      this.setState({
        isSingleMode: true,
        hidePeopleLable: true,
        showLocalStream: true,
      });
    } else if (mode == spotlightMode) {
      channelLastN = -1;
      this.setState({
        isSingleMode: false,
        hidePeopleLable: false,
        showLocalStream: true,
      });
    } else {
      channelLastN = 4;
      this.setState({
        isSingleMode: false,
        hidePeopleLable: true,
        hideUserThumbList: false,
        showLocalStream: false,
      });
    }

    if (this.props.isAudioOnly) {
    } else {
      this.setState({ hideAudioModeButton: true });
    }
  };

  init = (sessionID) => {
    roomName = sessionID;
    JitsiMeetJS.init(initOptions);
    JitsiMeetJS.setLogLevel(JitsiMeetJS.logLevels.ERROR);
    this.makeConnection();
  };

  makeConnection() {
    Logger.log(TAG, "Make connection region = ", this.state.region);
    Logger.log(TAG, "LastN = ", channelLastN);
    var options = {
      hosts: {
        domain: this.getDomain(),
        muc: this.getMUC(),
        focus: this.getFocus(),
      },
      serviceUrl : this.getWebSocket(),
      bosh: this.getBosh(),
      clientNode: "http://jitsi.org/jitsimeet",
    };
    Logger.log(TAG, "Make connection options = ", options);
    connection = new JitsiMeetJS.JitsiConnection(null, null, options);
    connection.connect();
    this.addEventListener();
  }
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
  getDomain() {
    var domain = this.props.domain;
    Logger.log(TAG, "RTC Domain = ", domain);
    return domain;
  }

  getMUC() {
    var muc = "conference."+this.props.domain;
    Logger.log(TAG, "RTC MUC = ", muc);
    return muc;
  }

  getFocus(){
    var focus = "focus."+this.props.domain;
    Logger.log(TAG,"RTC Focus = ",focus)
    return focus;
  }

  getBosh() {
    var bosh = "https://xmpp." + this.props.domain + "/http-bind";
    Logger.log(TAG, "RTC Bosh = ", bosh);
    return bosh;
  }

  getWebSocket(){
    var websocket = "wss://xmpp."+this.props.domain+"/xmpp-websocket"
    Logger.log(TAG,"RTC websocket = ", websocket)
    return websocket;
  }

  addNativeEventListner() {
    //this.eventListener = new NativeEventEmitter(AudioMode);
    eventEmitter.addListener("CallEvent", (type) => {
      Logger.log("Native received : ", type);
      if (type.type == "NativeEndCall") {
        this.onHangUp();
      }
    });
  }

  addEventListener() {
    connection.addEventListener(
      JitsiMeetJS.events.connection.CONNECTION_ESTABLISHED,
      this.onConnectionSuccess
    );
    connection.addEventListener(
      JitsiMeetJS.events.connection.CONNECTION_FAILED,
      this.onConnectionFailed
    );
    // connection.addEventListener(
    //   JitsiMeetJS.events.connection.CONNECTION_DISCONNECTED,
    //   this.onDisconnect
    // );
  }

  onConnectionSuccess = () => {

    const confOptions = {
      openBridgeChannel: "websocket",
      enableNoisyMicDetection: true,
      channelLastN: 10,
      websocketKeepAlive: -1,
      testing: {
        octo: { probability: 1 },
      },
      e2eping: { pingInterval: -1 },
      deploymentInfo: {
        userRegion: this.props.REGION,
      },
      enableTalkWhileMuted: true,
    };
    Logger.log(TAG, "confOptions Called......", confOptions);
    room = connection.initJitsiConference(roomName, confOptions);
    window.room = room;
    Logger.log(TAG, "onConnectionSuccess Called......", room);
    const myUserID = room.myUserId() || "";
    const users = [...this.state.users];

    const localUser = users[0];
    users[0] = {
      ...localUser,
      id: myUserID,
    };
    this.setState({
      users,
      dominantSpeakerId: room.lastDominantSpeaker || myUserID,
    });
    room.on(JitsiMeetJS.events.conference.TRACK_ADDED, this.onRemoteTrack);

    room.on(
      JitsiMeetJS.events.conference.TRACK_REMOVED,
      this.onRemoteTrackRemoved
    );
    room.on(
      JitsiMeetJS.events.conference.CONFERENCE_JOINED,
      this.onConferenceJoined
    );
    room.on(JitsiMeetJS.events.conference.USER_JOINED, this.onUserJoined);
    room.on(JitsiMeetJS.events.conference.USER_LEFT, this.onUserLeft);
    room.on(
      JitsiMeetJS.events.conference.TRACK_MUTE_CHANGED,
      this.onTrackMuted
    );
    room.on(
      JitsiMeetJS.events.conference.DOMINANT_SPEAKER_CHANGED,
      this.onDominantSpeakerChanged
    );
    room.on(
      JitsiMeetJS.events.connectionQuality.REMOTE_STATS_UPDATED,
      this.onRemoteStatsUpdated
    );
    room.on(
      JitsiMeetJS.events.connectionQuality.LOCAL_STATS_UPDATED,
      this.onLocalStatsUpdated
    );

    room.on(
      JitsiMeetJS.events.conference.PARTICIPANT_CONN_STATUS_CHANGED,
      (id, status) => {
        const users = this.state.users.map((user) => {
          if (user.id === id) {
            return {
              ...user,
              status,
            };
          }

          return user;
        });

        this.setState({ users });
      }
    );
    room.on(
      JitsiMeetJS.events.conference.LAST_N_ENDPOINTS_CHANGED,
      this.onLastNChanged
    );

    room.addCommandListener(SCREEN_SHARING_STARTED, ({ value }) => {
      this.setState({ pinnedUserId: value });
      if (room) {
        room.pinParticipant(value);
      }
    });

    room.addCommandListener(SCREEN_SHARING_STOPPED, ({ value }) => {
      if (value === this.state.pinnedUserId) {
        room.pinParticipant(null);
        this.setState({
          pinnedUserId: null,
        });
      }
    });
    room.setDisplayName(this.props.RTCUser.name);
    room.sendCommand(USER_AVATAR_URL, { value: this.props.RTCUser.avatar });
    room.sendCommand(USER_UID, { value: this.props.RTCUser.uid });
    room.sendCommand(USER_JOINED_AT, { value: Date.now() });

    room.addCommandListener(USER_JOINED_AT, (data, from) => {
      Logger.log(TAG, "User joinned at = ", data.value);
      const users = this.state.users.map((user) => {
        if (user.id === from) {
          return {
            ...user,
            joinedAT: data.value,
          };
        }
        return user;
      });

      this.setState({
        users,
      });
    });

    room.addCommandListener(USER_UID, (data, from) => {
      const users = this.state.users.map((user) => {
        if (user.id === from) {
          if (!user.uid) {
            Logger.log(TAG, "User uid is unique user ", this.props.RTCUser.uid);
            //eventEmitter.emit("CallEvent", { type: "onUserJoinned", uid : data.value });
          }
          return {
            ...user,
            uid: data.value,
          };
        }
        return user;
      });

      this.setState({
        users,
      });
    });
    room.addCommandListener(USER_AVATAR_URL, (data, from) => {
      const users = this.state.users.map((user) => {
        if (user.id === from) {
          return {
            ...user,
            avatar: data.value,
          };
        }
        return user;
      });

      this.setState({
        users,
      });
    });
    room.setReceiverVideoConstraint(180);
    room.join();
  };

  onLastNChanged = (leavingIds, enteringIds) => {
    Logger.log(TAG, "LastN LeavingIDs...  ", leavingIds);
    Logger.log(TAG, "LastN enteringIds...  ", enteringIds);
    let { users } = this.state;
    let isChanged = false;
    users.map((user) => {
      if (leavingIds.includes(user.id)) {
        isChanged = true;
        user.showVideo = false;
      } else if (enteringIds.includes(user.id)) {
        isChanged = true;
        user.showVideo = true;
      }
    });

    if (isChanged) {
      this.setState({ users });
    }
  };

  onLocalStatsUpdated = (stats) => {
    Logger.log(TAG, "onLocalStatsUpdated updated : ", stats);
    let users = [...this.state.users];
    users[0].stats = stats;
    this.setState({ users });
    this.setState({ resolutions: stats.resolution });
  };
  onRemoteStatsUpdated = (id, stats) => {
    Logger.log(TAG, "onRemoteStatsUpdated updated : ", id);
    let users = [...this.state.users];
    users.map((user) => {
      if (user.id == id) {
        user.stats = stats;
      }
    });
    this.setState({ users });
  };
  onConnectionFailed() {
    Logger.log(TAG, "onConnectionFailed Called......");
  }
  //TODO: need to uncoment this function ? :jitwar
  onDisconnect() {
    Logger.log(TAG, "onDisconnect Called......");
    // connection.removeEventListener(
    //   JitsiMeetJS.events.connection.CONNECTION_ESTABLISHED,
    //   this.onConnectionSuccess
    // );
    // connection.removeEventListener(
    //   JitsiMeetJS.events.connection.CONNECTION_FAILED,
    //   this.onConnectionFailed
    // );
    // connection.removeEventListener(
    //   JitsiMeetJS.events.connection.CONNECTION_DISCONNECTED,
    //   this.onDisconnect
    // );
  }

  onLastNEndPointsChanged = (leavingIds, enteringIds) => {
    Logger.log(TAG, "Leaving IDS ...  ", leavingIds);
    Logger.log(TAG, "Entering IDS ...  ", enteringIds);
  };

  onDominantSpeakerChanged = (id) => {
    Logger.log(TAG, "onDominantSpeakerChanged id ...  ", id);
    Logger.log(
      TAG,
      "Participants who pinned me...  ",
      this.participantsWhoPinnedMe
    );
    if (!id) {
      return;
    }

    if (this.state.users[0].id !== id) {
      this.setState({
        dominantSpeakerId: id,
      });
      if (!this.state.pinnedUserId) {
        room.pinParticipant(id);
      }
    }
  };

  onUserJoined = (id, remoteUser) => {
    Logger.log(TAG, "onUserJoined", remoteUser.getDisplayName());
    const user = {
      id: remoteUser.getId(),
      uid: null,
      joinedAT: undefined,
      name: remoteUser.getDisplayName(),
      avatar: null,
      status: remoteUser._connectionStatus,
      isAudioMuted: false,
      isVideoMuted: false,
      videoTrack: undefined,
      audioTrack: undefined,
      isLocalUser: false,
    };
    this.setState({
      users: [...this.state.users, user],
    });
  };

  onUserLeft = (id, user) => {
    Logger.log(TAG, "onUserLeft called ", "");
    //eventEmitter.emit("CallEvent", { type: "onUserJoinned", uid : data.value });
    const users = this.state.users.map((u) => {
      if (user._id === u.id) {
        Logger.log(TAG, "User letf with UID ==>", u.uid);
        eventEmitter.emit("CallEvent", { type: "onUserLeft", uid: u.uid });
      }
      //return user;
    });

    let usersData = this.state.users.filter((u) => u.id !== user._id);
    console.log(TAG, "onUserLeft after .....", usersData);

    if (usersData.length <= 1) {
      eventEmitter.emit("CallEvent", { type: "EndCall" });
    }

    this.setState({
      users: usersData,
    });
  };

  onTrackMuted = (track) => {
    Logger.log(TAG, "onTrackMuted ...... ", track);
    let id = track.ownerEndpointId;
    if (track.isLocal()) {
      id = this.state.users[0].id;
    }

    const users = this.state.users.map((user) => {
      if (user.id === id) {
        if (track.getType() === "video") {
          return {
            ...user,
            videoTrack: track,
            isVideoMuted: track.isMuted(),
          };
        }
        return {
          ...user,
          audioTrack: track,
          isAudioMuted: track.isMuted(),
        };
      }

      return user;
    });

    this.setState({
      users,
    });
  };

  onRemoteTrack = (track) => {
    Logger.log(TAG, "onRemoteTrack ...... ", track);
    if (track.isLocal()) {
      return;
    }

    const userId = track.ownerEndpointId;

    if (isScreenSharingTrack(track)) {
      this.setState({ pinnedUserId: userId });
    } else if (room.lastDominantSpeaker === userId) {
      this.setState({ dominantSpeakerId: userId });
    }

    const users = this.state.users.map((user) => {
      if (user.id === userId) {
        if (track.type === "video") {
          return {
            ...user,
            videoTrack: track,
            isVideoMuted: track.muted,
          };
        } else {
          return {
            ...user,
            audioTrack: track,
            isAudioMuted: track.muted,
          };
        }
      }

      return user;
    });

    this.setState({
      users,
    });
  };

  onRemoteTrackRemoved = (track) => {
    Logger.log(TAG, "onRemoteTrackRemoved ...... ", track);
    const users = this.state.users.map((user) => {
      if (user.id === track.ownerEndpointId) {
        if (track.type === "video") {
          return {
            ...user,
            videoTrack: undefined,
          };
        } else {
          return {
            ...user,
            audioTrack: undefined,
          };
        }
      }

      return user;
    });

    this.setState({ users });
  };

  onConferenceJoined = async () => {
    isJoined = true;

    const { localUser } = this;

    if (localUser.audioTrack) {
      room.addTrack(localUser.audioTrack);
    }

    if (localUser.videoTrack) {
      await room.addTrack(localUser.videoTrack);
    }
  };

  createLocalTracks = () => {
    if (this.props.isAudioOnly) {
      JitsiMeetJS.createLocalTracks({
        devices: ["audio"],
      }).then(this.onLocalTracks);
    } else {
      if (this.state.isSingleMode) {
        JitsiMeetJS.createLocalTracks({
          devices: ["audio", "video"],
          facingMode: "user",
          // resolution: 720,
          // constraints: {
          //   video: {
          //     height: {
          //       ideal: 720,
          //       max: 1080,
          //       min: 180,
          //     },
          //     maxFps: 15,
          //   },
          // },
        })
          .then(this.onLocalTracks)
          .catch((error) => {
            JitsiMeetJS.createLocalTracks({
              devices: ["audio"],
              facingMode: "user",
            }).then(this.onLocalTracks);
          });
      } else {
        Logger.log(TAG,"Selected video stream getting created.....")
        JitsiMeetJS.createLocalTracks({
          devices: ["audio", "video"],
          facingMode: "user",
          resolution: 360,
          constraints: {
            video: {
              height: {
                ideal: 360,
                max: 360,
                min: 180,
              },
              maxFps: 15,
            },
          },
        })
          .then(this.onLocalTracks)
          .catch((error) => {
            JitsiMeetJS.createLocalTracks({
              devices: ["audio"],
              facingMode: "user",
            }).then(this.onLocalTracks);
          });
      }
    }
  };

  onLocalTracks = (tracks, isResumedFromScreenSharing) => {
    for (let i = 0; i < tracks.length; i++) {
      if (tracks[i].getType() === "video") {
        const users = [...this.state.users];
        const localUser = users[0];
        if (localUser.isVideoMuted || this.state.isVideoMuted) {
          tracks[i].mute();
        }
        users[0] = {
          ...localUser,
          videoTrack: tracks[i],
        };
        this.setState({
          users,
        });

        // To handle Chrome/extension's stop button click
        if (isScreenSharingTrack(tracks[i])) {
          tracks[i].track.onended = () => {
            this.stopScreenShare();
          };
        }
      }
      if (tracks[i].getType() === "audio") {
        const users = [...this.state.users];
        const localUser = users[0];
        if (localUser.isAudioMuted || this.state.isAudioMuted) {
          tracks[i].mute();
        }
        users[0] = {
          ...localUser,
          audioTrack: tracks[i],
        };
        this.setState({
          users,
        });
      }
      if (isJoined) {
        room.addTrack(tracks[i]);
      }
    }
  };

  toggleVideoButton = () => {
    const { localUser } = this;
    Logger.log(TAG, "Local user Video Track muted ? ", localUser);
    if (localUser.videoTrack) {
      if (this.state.isVideoMuted) {
        localUser.videoTrack.unmute();
        this.setState({ isVideoMuted: false });
      } else {
        localUser.videoTrack.mute();
        this.setState({ isVideoMuted: true });
      }
    }
  };

  toggleAudioButton = () => {
    const { localUser } = this;
    if (localUser.audioTrack) {
      if (this.state.isAudioMuted) {
        localUser.audioTrack.unmute();
        this.setState({ isAudioMuted: false });
      } else {
        localUser.audioTrack.mute();
        this.setState({ isAudioMuted: true });
      }
    }
  };

  muteAudio = () => {
    const { localUser } = this;
    if (localUser.audioTrack) {
      localUser.audioTrack.mute();
    }
  };

  unmuteAudio = () => {
    const { localUser } = this;
    if (localUser.audioTrack) {
      localUser.audioTrack.unmute();
    }
  };

  pauseVideo = () => {
    const { localUser } = this;
    if (localUser.videoTrack) {
      localUser.videoTrack.mute();
    }
  };

  unPauseVideo = () => {
    const { localUser } = this;
    if (localUser.videoTrack) {
      localUser.videoTrack.unmute();
    }
  };

  onEndCallButtonCLicked = () => {
    CometChatRTC.log("onEndCallButtonCLicked");
    this.unload();
    window.postMessage(
      window.parent.postMessage(
        JSON.stringify({
          type: "hangup",
          message: "Message text from iframe.",
        }),
        "*"
      ),
      "*"
    );
  };

  disposeLocalVideoTrack = async () => {
    const { localUser } = this;
    if (localUser.videoTrack) {
      try {
        await localUser.videoTrack.dispose();
      } catch (e) {}
    }
    if (localUser.audioTrack) {
      try {
        await localUser.audioTrack.dispose();
      } catch (e) {}
    }
  };

  createScreenShareTrack = async () => {
    await this.disposeLocalVideoTrack();
    JitsiMeetJS.createLocalTracks({
      devices: ["audio", "desktop"],
      resolution: 360,
      constraints: {
        video: {
          height: {
            ideal: 360,
            max: 720,
            min: 180,
          },
          maxFps: 15,
        },
      },
    })
      .then((tracks) => {
        this.onLocalTracks(tracks);
        this.setState({
          isScreenShareOn: true,
        });
        room.sendCommand(SCREEN_SHARING_STARTED, {
          value: this.localUser.id,
        });
      })
      .catch((error) => {
        this.stopScreenShare();
        //Logger.log("Create Local error..");
      });
  };

  stopScreenShare = async () => {
    await this.disposeLocalVideoTrack();
    this.createLocalTracks();
  };

  onScreenShareClicked = () => {
    if (this.state.isScreenShareOn) {
      this.stopScreenShare();
    } else {
      this.createScreenShareTrack();
    }
  };

  onDisplayModeChange = () => {
    this.setState((state) => ({
      isGridMode: !state.isGridMode,
    }));
  };

  onUserItemClicked = (user) => {
    this.setState({
      dominantSpeakerId: user.id,
    });
  };

  //TODO: remove this function
  // renderLocalVideo = () => {
  //   const { users } = this.state;
  //   Logger.log(TAG, "Users ===>  ", users);
  //   const localUser = users[0];
  //   Logger.log(TAG, "renderLocalVideo for .... ", localUser.videoTrack);
  //   if (localUser.videoTrack) {
  //     Logger.log(TAG, "local stream = ", localUser);
  //     return (
  //       <RTCView
  //         key={"localmedia"}
  //         objectFit="cover"
  //         streamURL={localUser.videoTrack.getOriginalStream().toURL()}
  //         style={{ height: 90, width: 90, borderRadius: 5 }}
  //         zOrder={2}
  //       />
  //     );
  //   } else {
  //     Logger.log(TAG, "No local video track", localUser);
  //     return this.renderthumbAvatarView(localUser);
  //   }
  // };

  // renderAvatarView = (user) => {
  //   return (
  //     <View
  //       style={{
  //         height: "100%",
  //         width: "100%",
  //         alignItems: "center",
  //         justifyContent: "center",
  //         flexDirection: "column",
  //       }}
  //     >
  //       {user.avatar ? (
  //         <Image
  //           style={{ width: 110, height: 110 }}
  //           source={{ uri: user.avatar }}
  //         />
  //       ) : (
  //         <TextAvatar
  //           backgroundColor={"#4d4c4f"}
  //           textColor={"#FFF"}
  //           size={100}
  //           type={"circle"} // optional
  //         >
  //           {user.name}
  //         </TextAvatar>
  //       )}

  //       <Text style={{ fontSize: 18, color: "#fff", padding: 20 }}>
  //         {user.name}
  //       </Text>
  //     </View>
  //   );
  // };

  //TODO: need to remove this function
  // renderthumbAvatarView(user) {
  //   return (
  //     <View
  //       style={{
  //         height: 90,
  //         width: 90,
  //         alignItems: "center",
  //         justifyContent: "center",
  //       }}
  //     >
  //       {user.avatar ? (
  //         <Image
  //           style={{ width: 60, height: 60 }}
  //           source={{ uri: user.avatar }}
  //         />
  //       ) : (
  //         <TextAvatar
  //           backgroundColor={"#4d4c4f"}
  //           textColor={"#FFF"}
  //           size={50}
  //           type={"circle"}
  //         >
  //           {" "}
  //           {user.name}
  //         </TextAvatar>
  //       )}
  //     </View>
  //   );
  // }
  //TODO: need to remove this function
  renderSingleMainVideo = () => {
    const { users } = this.state;
    Logger.log(TAG, "Single Users = ", users);
    var maxTimestamp = 0;
    this.mainVideoSpeaker = undefined;
    Logger.log(TAG, "Main video speaker = ", this.mainVideoSpeaker);
    if (users.length > 1) {
      const uzz = this.state.users.map((u) => {
        if (this.mainVideoSpeaker == undefined) {
          this.mainVideoSpeaker = users[1];
        }
        if (
          !u.isLocalUser &&
          u.joinedAT != undefined &&
          u.joinedAT >= this.mainVideoSpeaker.joinedAT
        ) {
          this.mainVideoSpeaker = u;
        }
      });

      if (this.mainVideoSpeaker === undefined) {
        this.mainVideoSpeaker = users[1];
      }

      Logger.log(TAG, "SingleMainVideoSpeaker = ", this.mainVideoSpeaker);
      if (
        this.mainVideoSpeaker &&
        (this.mainVideoSpeaker.isVideoMuted ||
          this.mainVideoSpeaker.status !== "active")
      ) {
        return this.renderAvatarView(this.mainVideoSpeaker);
      }
      if (this.mainVideoSpeaker.videoTrack) {
        return (
          <RTCView
            key={this.mainVideoSpeaker.id}
            objectFit={"cover"}
            streamURL={this.mainVideoSpeaker.videoTrack
              .getOriginalStream()
              .toURL()}
            style={localStyles.full_height_width}
            zOrder={1}
          />
        );
      } else {
        return this.renderAvatarView(this.mainVideoSpeaker);
      }
    }
  };

  renderMainVideo = () => {
    // Logger.log(TAG,"renderMainVideo called.... ",'')
    const { users, dominantSpeakerId, pinnedUserId } = this.state;
    const localUser = users[0];
    const dominantUser =
      users.find((user) => user.id === dominantSpeakerId) || localUser;
    const pinnedUser = users.find((user) => user.id === pinnedUserId);
    this.mainVideoSpeaker = dominantUser;
    if (dominantUser) {
      let dominantVideoTrack = dominantUser.videoTrack;
      if (pinnedUser && pinnedUser.videoTrack) {
        dominantVideoTrack = pinnedUser.videoTrack;
        this.mainVideoSpeaker = pinnedUser;
      }

      const isTrackOfTypeScreenSharing = isScreenSharingTrack(
        dominantVideoTrack
      );
      try {
        var ids = [this.mainVideoSpeaker.id];
        room.selectParticipants(ids);
        Logger.log(TAG, "selectParticipants ids ", room.selectParticipants);
      } catch (err) {
        Logger.log(TAG, "selectParticipants ids error : ", err);
      }
      console.log("this.mainVideoSpeaker", this.mainVideoSpeaker);
      if (
        this.mainVideoSpeaker &&
        (this.mainVideoSpeaker.isVideoMuted ||
          this.mainVideoSpeaker.status !== "active" ||
          this.state.isAudioOnly)
      ) {
        return (
          <View style={localStyles.mainVideoAvatarMainView}>
            <TextAvatar
              backgroundColor={"#4d4c4f"}
              textColor={"#FFF"}
              size={100}
              type={"circle"} // optional
            >
              {this.mainVideoSpeaker.name}
            </TextAvatar>
            <Text style={localStyles.mainVideoName}>
              {this.mainVideoSpeaker.name}
            </Text>
            <AudioLevelIndicator track={this.mainVideoSpeaker.audioTrack} />
            <RTCView
              key={this.mainVideoSpeaker.id}
              objectFit="cover"
              streamURL={this.mainVideoSpeaker.audioTrack
                ?.getOriginalStream()
                .toURL()}
              style={localStyles.zero_height_width}
              zOrder={-10}
            />
          </View>
        );
      }

      // Logger.log(TAG,"renderMainVideo with dominant video track.... ",dominantVideoTrack)
      if (dominantVideoTrack) {
        if (dominantVideoTrack.isLocal()) {
          return (
            <View style={localStyles.main_video_video_track_view}>
              <RTCView
                mirror={this.state.isVideoMirrored}
                key={dominantSpeakerId}
                objectFit="contain"
                streamURL={dominantVideoTrack.getOriginalStream().toURL()}
                style={localStyles.full_height_width}
                zOrder={-20}
              />
              <Text style={localStyles.main_video_track_name}>
                {this.mainVideoSpeaker.name}
              </Text>
            </View>
          );
        } else {
          return (
            <View style={localStyles.main_video_video_track_view}>
              <RTCView
                key={dominantSpeakerId}
                objectFit="contain"
                streamURL={dominantVideoTrack.getOriginalStream().toURL()}
                style={localStyles.full_height_width}
                zOrder={-20}
              />

              <Text style={localStyles.main_video_track_name}>
                {this.mainVideoSpeaker.name}
              </Text>
            </View>
          );
        }
      }
    }
  };

  pinUser = (user) => {
    this.setState({
      pinnedUserId: user.id,
    });
    room.pinParticipant(user.id);
  };

  unPinUser = () => {
    room.pinParticipant(null);

    this.setState({
      pinnedUserId: null,
    });
  };

  onDoubleTap() {
    const { users } = this.state;
    const localUser = users[0];

    if (localUser.videoTrack) {
      localUser.videoTrack._switchCamera();
      this.setState({
        isVideoMirrored: !this.state.isVideoMirrored,
      });
    }
  }

  onAudioModeChanged = (mode) => {
    Logger.log(TAG, "Audio Mode = ", this.state.audioMode);
    if (this.state.audioMode !== mode) {
      this.setState({
        audioMode: mode,
      });
    }
  };

  onPeopleClicked = () => {
    Logger.log(TAG, "onPeopleClicked");
    this.setState({
      showUserPopUP: !this.state.showUserPopUP,
    });
  };

  showPeoplePopup = () => {
    Logger.log(TAG, "ShowPeople Popup");
    this.setState({
      showUserPopUP: true,
      hidePeopleLable: true,
    });
  };

  hidePeoplePopup = () => {
    Logger.log(TAG, "ShowPeople Popup");
    this.setState({
      showUserPopUP: false,
      hidePeopleLable: false,
    });
  };

  renderPeopleButton() {
    if (!this.state.hideVideoSwitchButton) {
      return (
        <TouchableOpacity onPress={this.onPeopleClicked}>
          <View style={localStyles.lt_button}>
            <Image
              style={localStyles.lt_control_icon}
              source={require("./icons/icon_people.png")}
            />
          </View>
        </TouchableOpacity>
      );
    }
  }

  // renderVideoThumbItem(userItem) {
  //   Logger.log(TAG, "Render View thumb item : ", userItem);

  //   if (userItem.item.videoTrack) {
  //     return (
  //       <View
  //         style={{
  //           height: 100,
  //           width: 100,
  //           backgroundColor: "#000",
  //           margin: 5,
  //           zIndex: 5,
  //         }}
  //       >
  //         {/* <RTCView
  //                 key={userItem.item.id}
  //                 objectFit="cover"
  //                 streamURL={userItem.item.videoTrack.getOriginalStream().toURL()}
  //                 style={{ height: 100, width: 100 , borderRadius : 5 }}
  //                 zOrder={5}
  //               />   */}
  //       </View>
  //     );
  //   }
  // }

  renderUserItem = (userItem) => {
    Logger.log(TAG, "Render User item : ", userItem);
    if (userItem) {
      var user = userItem.item;
      var index = userItem.index;
      var username = user.name;
      var isPinned;

      if (index == 0) {
        username += " (you)";
      }

      if (user.id === this.state.pinnedUserId) {
        isPinned = true;
      } else {
        isPinned = false;
      }

      return (
        <TouchableOpacity
          onPress={() => {
            if (user.id === this.state.pinnedUserId) {
              this.unPinUser();
            } else {
              this.pinUser(user);
            }
          }}
          rippleColor="rgba(0, 0, 0, .20)"
        >
          <View style={localStyles.user_item_pinned}>
            {/* <View style={localStyles.lt_button}><Image style={localStyles.lt_control_icon} source={require('./icons/icon_person.png')}/></View> */}
            <TextAvatar
              backgroundColor={"#4d4c4f"}
              textColor={"#FFF"}
              size={40}
              type={"circle"} // optional
            >
              {username}
            </TextAvatar>
            <Text style={localStyles.user_item_user_name}>
              {username}
              {this.state.resolutions[user.id]
                ? Object.values(this.state.resolutions[user.id])[0].width +
                  "X" +
                  Object.values(this.state.resolutions[user.id])[0].height
                : ""}
            </Text>
            <View style={localStyles.user_item_icon_container}>
              {user.isAudioMuted ? (
                <View style={localStyles.lt_small_button}>
                  <Image
                    style={localStyles.user_item_icons}
                    source={require("./icons/icon_audio_mute.png")}
                  />
                </View>
              ) : null}
              {user.isVideoMuted ? (
                <View style={localStyles.lt_small_button}>
                  <Image
                    style={localStyles.user_item_icons}
                    source={require("./icons/icon_video_mute.png")}
                  />
                </View>
              ) : null}

              {user.status === "interrupted" ? (
                <View style={localStyles.lt_small_button_red}>
                  <Image
                    style={localStyles.user_item_icons}
                    source={require("./icons/icon_alert.png")}
                  />
                </View>
              ) : null}

              {isPinned ? (
                <View style={localStyles.lt_small_button_blue}>
                  <Image
                    style={localStyles.user_item_icons}
                    source={require("./icons/icon_pin.png")}
                  />
                </View>
              ) : null}
            </View>
          </View>
        </TouchableOpacity>
      );
    }
  };

  renderEndCallButton() {
    if (this.state.hideEndCallButton) {
      return null;
    } else {
      return (
        <View style={localStyles.button_container}>
          <HangUpButton onHangUp={this.onHangUp} />
          {/* <Text style={localStyles.color_white}>end</Text> */}
        </View>
      );
    }
  }

  renderAudioPauseButton(localUser) {
    if (this.state.hideAudioMuteButton) {
      return null;
    } else {
      return (
        <View style={localStyles.button_container}>
          <AudioButton
            audioMute={this.state.isAudioMuted}
            onAudioMute={this.onAudioMute}
          />
          {/* {this.state.isAudioMuted ? (
            <Text style={localStyles.color_white}>audio off</Text>
          ) : (
            <Text style={localStyles.color_white}>audio on</Text>
          )} */}
        </View>
      );
    }
  }

  renderPauseVideoButton(localUser) {
    if (this.state.hidePauseVideoButton) {
      return null;
    } else {
      return (
        <View style={localStyles.button_container}>
          <VideoButton
            videoMute={this.state.isVideoMuted}
            onVideoMute={this.onVideoMute}
          />
          {/* {this.state.isVideoMuted ? (
            <Text style={localStyles.color_white}>video off</Text>
          ) : (
            <Text style={localStyles.color_white}>video on</Text>
          )} */}
        </View>
      );
    }
  }

  renderAudioModeButton(allow) {
    Logger.log(TAG, "HideAudioModeButton  = ", this.state.hideAudioModeButton);

    return (
      <View style={localStyles.button_container}>
        <AudioModeButton
          onAudioModeChanged={this.onAudioModeChanged}
          isAudioOnly={this.props.isAudioOnly}
          renderButton={true}
        />
        {/* <Text style={localStyles.color_white}>{this.state.audioMode}</Text> */}
      </View>
    );
  }

  renderCameraSwitchButton() {
    if (this.state.hideVideoSwitchButton) {
      return null;
    } else {
      return (
        <View style={localStyles.button_container}>
          <SwitchCameraButton onSwitchCamera={this.onDoubleTap} />
          {/* <Text style={localStyles.color_white}>flip</Text> */}
        </View>
      );
    }
  }

  onAudioMute() {
    Logger.log(TAG, "on Audio Muted clicked");
    this.toggleAudioButton();
  }

  onVideoMute() {
    Logger.log(TAG, "on video Muted clicked");
    this.toggleVideoButton();
  }

  onHangUp() {
    CometChatRTC.log("onHangUp clicked");
    this.unload();
    //eventEmitter.emit("CallEvent", { type: "EndCall" });
  }

  renderUserList = () => {
    let userlist = [...this.state.users];
    userlist.shift();
    if (
      this.state.pinnedUserId &&
      this.state.users[0].id != this.state.pinnedUserId
    ) {
      let pinnedIndex = null;
      for (let i = 0; i < userlist.length; i++) {
        if (userlist[i].id == this.state.pinnedUserId) {
          pinnedIndex = i;
          break;
        }
      }
      userlist.splice(pinnedIndex, 1);
    }

    userlist.sort(function (a, b) {
      return (a.showVideo || false) - (b.showVideo || false);
    });

    userlist = [this.state.users[0], ...userlist];
    let userVideos = [];

    for (let i = 0; i < (userlist.length > 2 ? 3 : userlist.length); i++) {
      let item = userlist[i];

      let index = i;

      let userStats = item.stats;
      let networkColor = userStats
        ? userStats.connectionQuality < 33
          ? "red"
          : userStats.connectionQuality < 66
          ? "yellow"
          : "green"
        : "";
      let username = item.name;
      let isPinned;

      if (index == 0) {
        username += " (you)";
      }

      if (item.id === this.state.pinnedUserId) {
        isPinned = true;
      } else {
        isPinned = false;
      }
      if (
        item.videoTrack &&
        !item.isVideoMuted &&
        !(item.showVideo === false)
      ) {
        userVideos.push(
          <TouchableOpacity
            onPress={() => {
              if (isPinned) {
                this.unPinUser();
              } else {
                this.pinUser(item);
              }
            }}
            rippleColor="rgba(0, 0, 0, .20)"
            style={localStyles.user_list_main_touchable}
          >
            <View
              style={{
                [this.state.isPortrait ? "width" : "height"]:
                  (this.state.isPortrait ? width : height) * 0.25,
                aspectRatio: 1,
                borderRadius: 5,
                [this.state.isPortrait
                  ? "marginHorizontal"
                  : "marginVertical"]: 5,
                backgroundColor: "#000",
              }}
            >
              <Text style={localStyles.user_list_user_name}>
                {item.name || ""}
              </Text>
              <View style={localStyles.user_list_icon_container}>
                {item.isAudioMuted ? (
                   <View style={localStyles.lt_small_button}>
                    <Image
                      style={localStyles.user_item_icons}
                      source={require("./icons/icon_audio_mute.png")}
                    />
                   </View>
                ) : null}

                {item.isVideoMuted ? (
                  <View style={localStyles.lt_small_button}>
                    <Image
                      style={localStyles.user_item_icons}
                      source={require("./icons/icon_video_mute.png")}
                    />
                  </View>
                ) : null}
                {isPinned ? (
                  <View style={localStyles.lt_small_button}>
                    <Text>Pin</Text>
                  </View>
                ) : null}
              </View>
              <RTCView
                key={item.id}
                objectFit="cover"
                streamURL={item.videoTrack?.getOriginalStream().toURL()}
                style={[localStyles.full_height_width, { borderRadius: 5 }]}
                zOrder={5}
              />
            </View>
            <View style={localStyles.user_list_network_container}>
              {/* <NetworkIcon fill={networkColor}/> */}
              <View
                style={[
                  localStyles.user_list_network_icon,
                  {
                    backgroundColor: networkColor,
                  },
                ]}
              />
            </View>
          </TouchableOpacity>
        );
      } else {
        userVideos.push(
          <TouchableOpacity
            onPress={() => {
              if (isPinned) {
                this.unPinUser();
              } else {
                this.pinUser(item);
              }
            }}
            rippleColor="rgba(0, 0, 0, .20)"
          >
            <View
              style={{
                [this.state.isPortrait ? "width" : "height"]:
                  (this.state.isPortrait ? width : height) * 0.25,
                aspectRatio: 1,
                alignItems: "center",
                justifyContent: "center",
                flexDirection: "column",
                borderRadius: 5,
                marginHorizontal: 5,
                backgroundColor: "#000",
              }}
            >
              <View style={localStyles.user_list_icon_container}>
                {item.isAudioMuted ? (
                  <View style={localStyles.lt_small_button}>
                    <Image
                      style={localStyles.user_item_icons}
                      source={require("./icons/icon_audio_mute.png")}
                    />
                  </View>
                ) : null}
                {item.isVideoMuted ? (
                  <View style={localStyles.lt_small_button}>
                    <Image
                      style={localStyles.user_item_icons}
                      source={require("./icons/icon_video_mute.png")}
                    />
                  </View>
                ) : null}
              </View>
              <TextAvatar
                backgroundColor={"#4d4c4f"}
                textColor={"#FFF"}
                size={30}
                type={"circle"} // optional
              >
                {item.name || ""}
              </TextAvatar>
              <Text style={localStyles.user_list_avatar_name}>
                {item.name || ""}
              </Text>
              <AudioLevelIndicator track={item.audioTrack} />
              <RTCView
                key={item.id}
                objectFit="cover"
                streamURL={item.audioTrack?.getOriginalStream().toURL()}
                style={localStyles.zero_height_width}
                zOrder={-1}
              />
            </View>
          </TouchableOpacity>
        );
      }
    }

    if (userlist.length > 2) {
      for (let i = 3; i < userlist.length - 1; i++) {
        if (userlist[i].videoTrack && !userlist[i].isVideoMuted) {
          userVideos.push(
            <RTCView
              key={userlist[i].id}
              objectFit="cover"
              streamURL={userlist[i].videoTrack?.getOriginalStream().toURL()}
              style={[localStyles.full_height_width, { borderRadius: 5 }]}
              zOrder={-1}
            />
          );
        } else if (userlist[i].audioTrack && !userlist[i].isAudioMuted) {
          userVideos.push(
            <RTCView
              key={userlist[i].id}
              objectFit="cover"
              streamURL={userlist[i].audioTrack?.getOriginalStream().toURL()}
              style={[localStyles.full_height_width, { borderRadius: 5 }]}
              zOrder={-1}
            />
          );
        }
      }
    }

    //  console.warn(userVideos.length)
    return userVideos;
  };

  render() {
    const { users, pinnedUserId } = this.state;
    const localUser = users[0];
    return (
      <View style={localStyles.mainContainer}>
        {/* <DoubleTap onDoubleTap={this.onDoubleTap}>
        // <View style={{ height: "100%", backgroundColor: "#282c34" }}> 
          <View style={{ height: "100%", backgroundColor: "#282c34" }}>
            {this.state.isSingleMode? this.renderSingleMainVideo() :this.renderMainVideo()}
          </View>
        </DoubleTap> */}
        <View style={localStyles.video_container}>
          {/* <DoubleTap onDoubleTap={this.onDoubleTap}> */}
          <View
            style={{
              position: "relative",
              backgroundColor: "#000",

              marginTop: 15,

              flex: 1,
              borderRadius: 10,
              marginHorizontal: this.state.isPortrait ? "5%" : 4,
            }}
          >
            {this.state.showUserList ? (
              <View
                style={{
                  position: "absolute",
                  top: 0,
                  bottom: 0,
                  width: "100%",
                  backgroundColor: "rgba(46,47,48,0.8)",
                  justifyContent: "center",
                  elevation: 10,
                  zIndex: 10,
                  justifyContent: "space-between",
                }}
              >
                <FlatList
                  style={{ width: "100%" }}
                  data={this.state.users}
                  keyExtractor={(user) => user.id}
                  renderItem={this.renderUserItem}
                />
                <TouchableOpacity
                  style={{ padding: 10, width: "100%", alignItems: "center" }}
                  onPress={() => this.setState({ showUserList: false })}
                >
                  <Text>Close</Text>
                </TouchableOpacity>
              </View>
            ) : null}
            <ReactNativeZoomableView
              maxZoom={1.5}
              minZoom={0.5}
              zoomStep={0.5}
              initialZoom={1}
              bindToBorders={true}
              style={{ height: "100%", width: "100%" }}
            >
              <View
                style={{
                  width: "100%",
                  height: "100%",
                  alignItems: "center",
                  justifyContent: "center",
                }}
              >
                {this.renderMainVideo()}
              </View>
            </ReactNativeZoomableView>
            {this.state.pinnedUserId ? (
              <TouchableOpacity
                style={{ position: "absolute", top: 10, right: 10 }}
                onPress={() => this.unPinUser()}
              >
                <Image
                  style={{ width: 30, height: 30 }}
                  source={require("./icons/icon_pin.png")}
                />
              </TouchableOpacity>
            ) : null}
          </View>
          {/* </DoubleTap> */}

          <View
            style={{
              height: this.state.isPortrait ? "18%" : "100%",
              width: this.state.isPortrait ? "100%" : "20%",
              paddingTop: "2%",
              paddingHorizontal: 10,
            }}
          >
            <View
              style={
                this.state.isPortrait
                  ? {
                      width: "100%",
                      flexDirection: "row",
                      justifyContent: "space-between",
                    }
                  : {
                      width: "100%",
                      height: "100%",
                    }
              }
            >
              <View style={{ flexDirection: "row" }}>
                {this.state.users.length >= 1 && this.renderUserList()}
              </View>
              {this.state.users.length > 3 ? (
                <TouchableOpacity
                  style={{ alignSelf: "flex-end" }}
                  onPress={() => {
                    this.setState({ showUserList: !this.state.showUserList });
                  }}
                >
                  <View
                    style={{
                      ...(this.state.isPortrait
                        ? { width: width * 0.15, height: width * 0.25 }
                        : { height: height * 0.15, width: width * 0.2 }),
                      alignSelf: "flex-end",
                      alignItems: "center",
                      justifyContent: "center",
                      flexDirection: "column",
                      borderRadius: 5,
                      marginHorizontal: 5,
                      backgroundColor: "#000",
                    }}
                  >
                    <Text style={{ fontSize: 18, color: "#fff", padding: 5 }}>
                      {this.state.users.length.toString()}
                    </Text>
                    <Text style={{ fontSize: 18, color: "#fff", padding: 5 }}>
                      users
                    </Text>
                  </View>
                </TouchableOpacity>
              ) : null}
            </View>
          </View>
        </View>
        {/* {this.state.showLocalStream ? <View
          style={{
            height: 90,
            width: 90,
            backgroundColor: "#000",
            position: "absolute",
            top: 0,
            margin : 5,
            borderRadius : 10,
            right: 0,
          }}
        >
          {this.state.users[0] ? this.renderLocalVideo() : ""}
        </View> : null} */}

        {/* {this.state.hidePeopleLable ? null : 
          <TouchableOpacity 
            onPress={this.showPeoplePopup}
            style={{
            height: 40,
            width: 50,
            backgroundColor: "#FFF",
            position: "absolute",
            top: 110,
            flexDirection: "row",
            borderTopLeftRadius: 10,
            borderBottomLeftRadius : 10,
            alignItems: 'center', justifyContent:'center',
            right: 0,
            zIndex: 10,
          }}>
            <Image
              style={localStyles.lt_control_icon}
              source={require("./icons/icon_people_black.png")}
            />
            <Text style = {{color:'#000',fontSize: 18}}>{this.state.users.length}</Text>
          </TouchableOpacity>
        } */}

        {/* {this.state.hideUserThumbList ? null : <View style={localStyles.container_thum_list}>
                                                  <ThumbnailView  users={this.state.users} pinnedUserId={pinnedUserId} pinUser = {this.pinUser} unPinUser={this.unPinUser} />
                                                </View>
        } */}

        {/* {this.state.showUserPopUP ? (
            <View style={localStyles.container_people}>
              <TouchableOpacity style = {{width: 35,height: 35,position: "absolute",right:0,zIndex: 10 ,backgroundColor: "#e5e5e5",borderRadius: 20,margin : 5,alignItems: "center",justifyContent: "center"}} 
              onPress={this.hidePeoplePopup}>
                <Image
                  style={{ width: 25, height: 25,margin :10 }}
                  source={require("./icons/icon_clear.png")}
                />
              </TouchableOpacity>

              <Text
                style={{
                  fontSize: 20,
                  fontWeight: "bold",
                  color: "#000",
                  padding: 10,
                }}
              >
              {" "}
              People({this.state.users.length}){" "}
              </Text>

              <FlatList
                style={{ width: "100%" }}
                data={this.state.users}
                keyExtractor={(user) => user.id}
                renderItem={this.renderUserItem}
              />
            </View>
        ) : null} */}

        {this.state.showButtonLayout ? (
          <View style={localStyles.container}>
            {/* <View style={localStyles.container_background} /> */}
            <View
              id="lt_control_container"
              style={localStyles.lt_control_container}
            >
              {this.props.isAudioOnly
                ? null
                : this.renderPauseVideoButton(localUser)}
              {this.renderAudioPauseButton(localUser)}

              {this.renderEndCallButton()}

              {this.renderAudioModeButton()}

              {/* {this.renderAudioModeButton()} */}

              {this.props.isAudioOnly ? null : this.renderCameraSwitchButton()}
            </View>
          </View>
        ) : null}
      </View>
    );
  }
}
