import React, { Component } from "react";
import { View } from "react-native";

const AUDIO_LEVEL_DOTS = 3;
import JitsiMeetJS from "lib-jitsi-meet/lib-jitsi-meet.min";

const CENTER_DOT_INDEX = Math.floor(AUDIO_LEVEL_DOTS / 2);

class AudioLevelIndicator extends Component {
  state = {
    audioLevel: 0,
  };
  _isMounted = false;

  setAudiolevel = (audioLevel) => {
    if (!this._isMounted) return;
    if (this.props.track && !this.props.track.isMuted()) {
      this.setState({ audioLevel });
    }
  };
  componentDidMount() {
    this._isMounted = true;
    if (this.props.track) {
      this.props.track.on(
        JitsiMeetJS.events.track.TRACK_AUDIO_LEVEL_CHANGED,
        this.setAudiolevel
      );
    }
  }

  componentDidUpdate(prevProps) {
    if (
      this.props.track !== prevProps.track &&
      prevProps.track &&
      this.props.track
    ) {
      prevProps.track.off(
        JitsiMeetJS.events.track.TRACK_AUDIO_LEVEL_CHANGED,
        this.setAudiolevel
      );

      this.props.track.on(
        JitsiMeetJS.events.track.TRACK_AUDIO_LEVEL_CHANGED,
        this.setAudiolevel
      );
    }
  }

  componentWillUnmount() {
    this._isMounted = false;
  }

  render() {
    // First make sure we are sensitive enough.
    const audioLevel = Math.min(this.state.audioLevel * 1.2, 1);
    // Let's now stretch the audio level over the number of dots we have.
    const stretchedAudioLevel = AUDIO_LEVEL_DOTS * audioLevel;
    const audioLevelDots = [];

    for (let i = 0; i < AUDIO_LEVEL_DOTS; i++) {
      const distanceFromCenter = CENTER_DOT_INDEX - i;
      const audioLevelFromCenter =
        stretchedAudioLevel - Math.abs(distanceFromCenter);
      const cappedOpacity = Math.min(1, Math.max(0, audioLevelFromCenter));
      let className;
console.log("cappedOpacity:",cappedOpacity)
      audioLevelDots.push(
        <View
          key={i}
          style={{
            opacity: 1,
            width: 5,
            height: cappedOpacity ? 5 + 10 * cappedOpacity : 5,
            borderRadius: 5,

            backgroundColor: "#388E8E",
            margin: 1,

            // transition: "opacity 0.25s ease-in-out",
            // "-moz-transition": "opacity 0.25s ease-in-out",
            // opacity: cappedOpacity,
          }}
        />
      );
    }

    return (
      <View
        style={{
          height: 16,

          alignItems: "center",
          flexDirection: "row",
        }}
      >
        {audioLevelDots}
      </View>
    );
  }
}

export default AudioLevelIndicator;
