import React, {Component} from 'react';
import { View, TouchableOpacity, ScrollView , Text ,Image } from 'react-native';
import { connect } from 'react-redux';
import {RTCView} from 'react-native-webrtc';
import { Platform } from '../react';
import Logger from 'react-native-cometchat-calling-component/lib/Logger';
import JitsiManager from 'react-native-cometchat-calling-component/lib/JitsiManager';
import TextAvatar from "./TextAvatar";

const TAG = 'ThumbnailView';
var pinnedUserId;
export default class ThumbnailView extends Component{
    constructor(props){
        super(props);
        this.onUserClicked = this.onUserClicked.bind(this);
        this.state = {
            pinnedUserId: null,
        }
    }

    onUserClicked = (user) => {
        if (user.id === pinnedUserId) {
            this.props.unPinUser();
            this.setState({pinnedUserId : null});
        } else {
            this.setState({pinnedUserId : user.id});
            this.props.pinUser(user);
        }
    };

    shouldComponentUpdate(nextProps, nextState) {
        pinnedUserId = nextProps.pinnedUserId;
        // if(nextProps.users === this.props.users){
        //     return false;
        // }
        return true;  
    }

    renderAvatarView(user){
        Logger.log(TAG,"Avatar = ",user.avatar);
        return <View style={{ height: 100, width: 100 , alignItems: 'center' , justifyContent: 'center' }} >
                    {user.avatar ? <Image style={{width: 60, height: 60}} source={{uri:user.avatar}} /> : <TextAvatar
                      backgroundColor={"#4d4c4f"}
                      textColor={"#FFF"}
                      size={50}
                      type={"circle"} // optional
                    >
                      {user.name}
                    </TextAvatar>}
                    
                </View>
    }

    renderThumbVideo(user){
        if(user.isVideoMuted || user.status !== "active"){
            return this.renderAvatarView(user);
        }else{
            return <RTCView
                    key={'Thumb'+user.id}
                    objectFit = "cover"
                    streamURL = {user.videoTrack.getOriginalStream().toURL()}
                    zOrder={2}
                    style={{ height: 100, width: 100 }}
                />
        }
        
    }

    render(){
        var showImageThumb = false;
        Logger.log(TAG,"New Users = ",this.props.users)
        if(showImageThumb){
            this.props.updateDominantVideoId('localVideo');
        }

        return(
            <ScrollView contentContainerStyle={{ flexGrow: 1 }}>
                {
                    this.props.users.map((user,index) => {   
                            var username,avatar;
                            return (
                                <View
                                  key={index}
                                  style={{
                                    height: 100,
                                    width: 100,
                                    margin : 5,
                                    marginStart :5,
                                    marginBottom : 30,
                                  }}
                                >
                                    
                                    <TouchableOpacity onPress={()=>{this.onUserClicked(user)}}>
                                        <View style={this.getPinnedItemStyle(user)}>
                                            <Text numberOfLines={1} ellipsizeMode='tail' style={this.style.name}>
                                                {user.name}
                                            </Text>
                                            
                                            {user.isAudioMuted ? <View style={{height:10,width:10,margin:5}}>
                                                                    <Image style={{height:15,width:15}} source={require('../icons/icon_audio_mute.png')}/>
                                                                 </View> : null}
                                        
                                        </View>
                                        <View style={{backgroundColor:'#000'}}>
                                            {user.videoTrack ? this.renderThumbVideo(user): this.renderAvatarView(user)}
                                        </View>
                                        
                                    </TouchableOpacity>
                                
                                </View>
                            )
                    })
                }
            </ScrollView>
        )
    }

    getPinnedItemStyle(user) {
        if(user.id == this.state.pinnedUserId){
            return {
                alignItems: 'center', 
                flexWrap: 'wrap',
                padding: 5,
                borderTopLeftRadius : 5,
                borderTopRightRadius :5,
                flexDirection: 'row',
                backgroundColor: '#000',
            }
        }else{
            return {
                alignItems: 'center', 
                flexWrap: 'wrap', 
                opacity:0.4,
                padding: 5,
                borderTopLeftRadius : 5,
                borderTopRightRadius :5,
                flexDirection: 'row',
                backgroundColor: '#000',
            }
        }
    }

    style = {
        lt_container : { display:'flex', flexDirection: 'column', alignItems: 'flex-end', bottom: 0 },
        container: Platform.OS === 'ios' ? { display: 'flex', flex: 1, zIndex: 1, alignSelf: 'flex-end' }  : { flex:1, zIndex:5 , marginBottom : 90},
        name: {
            color: "#fff",
            alignSelf:'center',
            justifyContent: 'center',
        },
        container_background:{
            alignItems: 'center', 
            flexWrap: 'wrap', 
            opacity:0.4,
            padding: 5,
            borderTopLeftRadius : 5,
            borderTopRightRadius :5,
            flexDirection: 'row',
            backgroundColor: '#000',
        },
    };
}

// function mapStateToProps (state) {
//     return {
//         localurl: state.url.localUrl,
//         localTracks: state.tracks.localTracks,
//         remoteTracks: state.tracks.remoteTracks,
//         dominantVideoId: state.tracks.dominantVideoId,
//         users : state.tracks.users,
//     }
// }

// function mapDispatchToProps(dispatch){
//     return{
//         updateDominantVideoId: (id) => dispatch(data.updateDominantVideoId(id))
//     }
// }

// export default connect(mapStateToProps, mapDispatchToProps)(ThumbnailView)