import Logger from "react-native-cometchat-calling-component/lib/Logger";
import JitsiManager from "react-native-cometchat-calling-component/lib/JitsiManager";

const TAG = 'Reducer Tracks';
const ADD_LOCAL_TRACK = 'ADD_LOCAL_TRACK';
const ADD_REMOTE_TRACK = 'ADD_REMOTE_TRACK';
const CHANGE_READY_STATE = 'CHANGE_READY_STATE';
const CHANGE_ROOM_NAME = 'CHANGE_ROOM_NAME';
const CHANGE_AUDIO_STATE = 'CHANGE_AUDIO_STATE';
const CHANGE_VIDEO_STATE = 'CHANGE_VIDEO_STATE';
const CONNECTION_FAILED = 'CONNECTION_FAILED';
const UPDATE_DOMINANT_VIDEO_ID = 'UPDATE_DOMINANT_VIDEO_ID';
const UPDATE_DOMINANT_VIDEO_TRACK = 'UPDATE_DOMINANT_VIDEO_TRACK';
const REMOVE_REMOTE_TRACK = 'REMOVE_REMOTE_TRACK';
const REMOVE_LOCAL_TRACKS = 'REMOVE_LOCAL_TRACKS';
const REMOVE_REMOTE_TRACKS = 'REMOVE_REMOTE_TRACKS';
const CHANGE_AUDIO_MODE = 'CHANGE_AUDIO_MODE';
const ADD_USER = 'ADD_USER';
const REMOVE_USER = 'REMOVE_USER';
const CLEAR_USERS = 'CLEAR_USERS';

const initialState = { 
  localTracks: [],
  remoteTracks: [],
  users:[],
  ready: false,
  room: '',
  audioMute: false,
  audioSpeaker: false,
  videoMute: false,
  connection_failed: 0,
  dominantVideoTrack : undefined,
  dominantVideoId: null
}

const Reducer = (state = initialState, action) => {

  const newState = { ...state };

  switch (action.type) {

    case REMOVE_REMOTE_TRACK:
      var id = action.payload;
      var index = state.remoteTracks.findIndex(track => track.ownerId == id);
      if(index != -1){
        var remoteTracks = [...state.remoteTracks];
        remoteTracks.splice(index,1);
        const newUpdatedState =  {...state, remoteTracks};
        return newUpdatedState;
      }
    break;

    case REMOVE_LOCAL_TRACKS:
      newState.localTracks = initialState.localTracks;
    break;

    case REMOVE_REMOTE_TRACKS:
      newState.remoteTracks = initialState.remoteTracks;
    break;

    case UPDATE_DOMINANT_VIDEO_ID:
      var id = action.payload;
      newState.dominantVideoId = id;
    break;

    case UPDATE_DOMINANT_VIDEO_TRACK:
      var track = action.payload;
      newState.dominantVideoTrack = track;
    break;

    case ADD_LOCAL_TRACK:
      var localTracksData = action.payload.localTracks; 
      if(localTracksData.length != 0 ){
        newState.localTracks = localTracksData;    
      }
    break;

    case ADD_USER:
        var user = action.payload.user;
        newState.users = [...state.users, user];
    break;

    case REMOVE_USER:
        var id = action.payload.userId;
        Logger.log(TAG,"Remove user called : ",id);
        var userdata = [...state.users];
        Logger.log(TAG,"Userdata = ",userdata)
        for (var i = 0; i < userdata.length; i++) {
          if (userdata[i].id === id) {
              userdata.splice(i, 1);
          }
        }
        if(userdata.length <= 1){
          JitsiManager.eventEmitter.emit('CallEvent', { type: 'EndCall' });
        }
        newState.users = userdata;
    break;

    case CLEAR_USERS:
        newState.users = [];
    break;
   
    case ADD_REMOTE_TRACK:
      var remoteTracksData = action.payload.remoteTracks;
      var id = remoteTracksData.ownerEndpointId;
      var type = remoteTracksData.type;
      var index = newState.users.findIndex(user => user.id == id);
      if(index != -1){
        var userdata = [...state.users];
        var user = userdata[index];
        Logger.log('User object = ',user)
        if(type === 'video'){
          user.videoTrack = remoteTracksData;
        }else{
          user.audioTrack = remoteTracksData;
        }
        userdata[index] = user;
        newState.users = userdata;
      }
      
    break;
    
    case CHANGE_READY_STATE:
      newState.ready = action.payload.ready;
    break;

    case CHANGE_AUDIO_STATE:
      newState.audioMute = action.payload.audioMute;
    break;

    case CHANGE_AUDIO_MODE:
      newState.audioSpeaker = action.payload.audioMode;
    break;

    case CHANGE_VIDEO_STATE:
      newState.videoMute = action.payload.videoMute;
    break;

    case CHANGE_ROOM_NAME:
      newState.room = action.payload.room;
    break;

    case CONNECTION_FAILED:
      newState.connection_failed = action.payload.status;

    default:
    break;
  }
  return newState; 
}

export default Reducer;
